############################################################
#
# System Function Definitions for hard disk and partitions
#
# Author: Martin Murth
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# CHANGES: 30/10/03 modified set_partitions() to use nlists
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# mdified set_partitions()                (2004-06-10)
############################################################

declaration template pro_declaration_functions_filesystem;

############################################################
#
# Common functions
#
############################################################

############################################################
#
# FUNCTION num_of_harddisks
#
############################################################

define function num_of_harddisks = {
   #
   # get resource harddisks
   #
   harddisks = value("/hardware/harddisks");
   #
   # count entries of resource harddisks
   #
   num_of_harddisks = length(harddisks);
   return (num_of_harddisks);
};


############################################################
#
# FUNCTION set_partitions
# takes a list of partitions and sets the machines
# partition table
#
# XXX old syntax deprechated XXX
# parameters: path(string),part_op(nlist)
#             part_op: size(long)   : size in bytes
#				(problem here with the units)
#                      disk(long)   : disk number (0,1,2..)
#                      type(string) : filesystem ("swap","ext3",etc)
#             (opt.)   ksopt(string): KS options(eg. "--grow")
#             (opt.)   quota(long)  : quota percentage (integer)
#
# new syntax
# set_partitions(block_device:string ,nlist("type",string, "size",long,
#                                            "mountpoint",string, ...),
#              block_device:string, nlist(...),
#              ...);
#
# changed to match the restrictions / possibilties of i386
# 2004-06-10 Martin Bock <bock@delta.ft.uam.es>
#
############################################################

define function set_partitions = {
   if ( argc != 1) error ("expecting precisely one argument");
   if ( !is_resource(argv[0])) error('not a resource: argv[0]');
   partition_table = argv[0];

   filesystems = value("/system/filesystems");
   num_of_partitions = length(partition_table);

   # for old calling of this function, XXX deprechated XXX
   if ( is_list(partition_table) ) {
      entry = 0;
      while (entry < num_of_partitions) {
         partition = partition_table[entry];

         if (length(partition) != 2) {
           error("wrong number of entries in partition line " +
           to_string(entry));
         };

         ksopt = undef;
         quota = undef;

         mountpoint  = partition[0];
         size        = partition[1]["size"];
         disk        = key(value("/hardware/harddisks"), partition[1]["disk"]);
         fs_type     = partition[1]["type"];
         if ( exists(partition[1]["ksopt"]) ) ksopt = partition[1]["ksopt"];
         if ( exists(partition[1]["quota"]) ) quota = partition[1]["quota"];

         if (!exists(position[disk])) position[disk] = 0;

         # set disk resource
         block_device = disk + to_string(position[disk] + 1);

         # set partition info
         filesystems["disks"][disk][block_device]["size"] = size;
         filesystems["disks"][disk][block_device]["type"] = fs_type;
         if (is_defined(ksopt)) {
            filesystems["disks"][disk][block_device]["ksopt"] = ksopt;
            };
         # set mount info
         filesystems["disks"][disk][block_device]["mountpoint"] = mountpoint;
         if ( is_defined(quota) ) {
            filesystems["disks"][disk][block_device]["quota"] = quota;
            };

         
         # jump to next partition
         position[disk] = position[disk] + 1;
         entry = entry + 1;
         };
      return(filesystems);
      };

   # for the new syntax
   if( is_nlist(partition_table) ) {
      partition = first(partition_table, block_device, settings);
      
      # traverse through partition table
      aaa = 0;
      while( aaa  < length(partition_table) ) {
         if ( !is_nlist(settings) ) {
            error("expecting nlist for partition settings");
            };

         # setting the partition information
         bbb = matches(block_device, '^([hs]d[a-z])([0-9]{1,2})$');
         disk = bbb[1];
         filesystems["disks"][disk][block_device] = settings;

         partition = next(partition_table, block_device, settings);
         aaa = aaa + 1;
         }; # finished traversing
      
      return(filesystems);
      };
   
   # when we arrive here somthing strange went wrong
   error("underermined error");
   };
