#######################################################################
#
# pro_declaration_functions_network
# functions related to "/system/network"
#
# RESPONSIBLE: Rafael A. Garcia Leiva <angel.leiva@uam.es>
# VERSION: 1.1.7
#
# created by Martin Bock <bock@delta.ft.uam.es>
#                                         (2004-06-27)
#
#######################################################################

declaration template pro_declaration_functions_network;

include pro_declaration_type_validation_functions;

# 
# makes defaults for gateway, netmask, broadcast for each network interface
#
define function set_interface_defaults = {
   # general checking
   if( argc != 1 ) error("precisely one argument expected");
   if( !is_nlist(argv[0]) ) error("expecting nlist as argument");
   if( !exists(argv[0]["netmask"]) || !is_ip(argv[0]["netmask"]) ) {
      error("default settings for netmask expected");
      };
   if( !exists(argv[0]["broadcast"]) || !is_ip(argv[0]["broadcast"]) ) {
      error("default settings for broadcast expected");
      };
   if( !exists(argv[0]["gateway"]) || !is_ip(argv[0]["gateway"]) ) {
      error("default settings for gateway expected");
      };

   # make defaults to all interfaces present in /system/network/interfaces
   interfaces = value("/system/network/interfaces"); 
   aaa = first(interfaces, ifname, ifsets);
   while(aaa) {
      if( !exists(ifsets["netmask"]) || !is_defined(ifsets["netmask"]) ) {
         ifsets["netmask"]    = argv[0]["netmask"];
         };
      if( !exists(ifsets["broadcast"]) || !is_defined(ifsets["broadcast"]) ) {
         ifsets["broadcast"]  = argv[0]["broadcast"];
         };
      if( !exists(ifsets["gateway"]) || !is_defined(ifsets["broadcast"]) ) {
         ifsets["gateway"]    = argv[0]["gateway"];
         };
      aaa = next(interfaces, ifname, ifsets);
      };

   # return the settings
   return(interfaces);
   };
